/*----------------------------------------------------------------------*
 * Display the date and time from a DS3231 or DS3232 RTC every second.  *
 * Display the temperature once per minute. (The DS3231 does a          *
 * temperature conversion once every 64 seconds. This is also the       *
 * default for the DS3232.)                                             *
 *                                                                      *
 * Set the date and time by entering the following on the Arduino       *
 * serial monitor:                                                      *
 *    year,month,day,hour,minute,second,                                *
 *                                                                      *
 * Where                                                                *
 *    year can be two or four digits,                                   *
 *    month is 1-12,                                                    *
 *    day is 1-31,                                                      *
 *    hour is 0-23, and                                                 *
 *    minute and second are 0-59.                                       *
 *                                                                      *
 * Entering the final comma delimiter (after "second") will avoid a     *
 * one-second timeout and will allow the RTC to be set more accurately. *
 *                                                                      *
 * No validity checking is done, invalid values or incomplete syntax    *
 * in the input will result in an incorrect RTC setting.                *
 *                                                                      *
 * Jack Christensen 08Aug2013                                           *
 *                                                                      *
 * Tested with Arduino 1.0.5, Arduino Uno, DS3231/Chronodot, DS3232.    *
 *                                                                      *
 * This work is licensed under the Creative Commons Attribution-        *
 * ShareAlike 3.0 Unported License. To view a copy of this license,     *
 * visit http://creativecommons.org/licenses/by-sa/3.0/ or send a       *
 * letter to Creative Commons, 171 Second Street, Suite 300,            *
 * San Francisco, California, 94105, USA.                               *
 * ------------------------------------------------------------------   *
 * Adaptation par gepeto@du-libre.org 28/10/14                          *
 * pour lecture d'une horloge DS3132 a travers un arduino mega328       *
 * et commande de relai bistable                                        *
 * pour une programmation de mise en route                              *
 *----------------------------------------------------------------------*/ 
 
#include <DS3232RTC.h>        //http://github.com/JChristensen/DS3232RTC
#include <Streaming.h>        //http://arduiniana.org/libraries/streaming/
#include <Time.h>             //http://playground.arduino.cc/Code/Time
#include <Wire.h>             //http://arduino.cc/en/Reference/Wire
#include <dht.h>


int pinButton = 8;
int RelayON = 11;  //Set coil//
int RelayOFF = 12; //Reset coil //
int stateRelay = LOW;
int stateButton;
int previous = LOW;
long time = 0;
long delaiButton = 5000;
int stayON = 5000; //stay on for 5000 ms
int rtcok=0; // i2c communication ok //
dht DHT; // capteur d'humidité //
#define DHT22_PIN 5


//Print an integer in "00" format (with leading zero),
//followed by a delimiter character to Serial.
//Input value assumed to be between 0 and 99.
void printI00(int val, char delim)
{
    if (val < 10) Serial << '0';
    Serial << _DEC(val);
    if (delim > 0) Serial << delim;
    return;
}
//print time to Serial
void printTime(time_t t)
{
    printI00(hour(t), ':');
    printI00(minute(t), ':');
    printI00(second(t), ' ');
}

//print date to Serial
void printDate(time_t t)
{
  printI00(day(t),'/');
  printI00(month(t),'/');
  printI00(year(t),0); 
    //printI00(day(t), 0);  //
    //Serial << monthShortStr(month(t)) << _DEC(year(t)); //
}


void printDateTime(time_t t)
{
    printDate(t);
    Serial << ' ';
    printTime(t);
}

void setup(void)
{
    pinMode(pinButton, INPUT);
    pinMode(RelayON , OUTPUT);
    pinMode(RelayOFF, OUTPUT); 
    Serial.begin(115200);
     
    //setSyncProvider() causes the Time library to synchronize with the
    //external RTC by calling RTC.get() every five minutes by default.
    setSyncProvider(RTC.get);
    //Serial << F("RTC Sync");//
    if (timeStatus() != timeSet) {
      Serial << F(" FAIL!");
      Serial << endl;
      rtcok=0;
    }
    else { 
       rtcok=1;
    }
    digitalWrite(RelayOFF, LOW);
    digitalWrite(RelayON, HIGH);  // Relay ON //
    delay(7);
    digitalWrite(RelayON, LOW);
    stateRelay = HIGH;    
    stateButton = HIGH;
    stateRelay = HIGH;
    previous = HIGH;
    time = millis();
   //enable alarm interrupts on match; also sets status flags A1F,A2F
    if (rtcok==1) {
      RTC.alarmInterrupt(1,1);//enable alarm 1 interrupt A1IE//
      RTC.alarmInterrupt(2,1);//enable alarm 2 interrupt A2IE//

  //enable or disable the square wave output,
  // no square wave (NONE) sets ITCN bit=1  
  //enables alarm interrupt on square wave pin set from A1F, A2F
  //digitalRead with INPUT_PULLUP; LOW is alarm interrupt
      RTC.squareWave(SQWAVE_NONE);
      RTC.alarmInterrupt(ALARM_1, true);      //assert the INT pin when Alarm1 occurs.
      RTC.alarmInterrupt(ALARM_2, true);     // Alarm2
    }

}

void loop(void)
{
// Gestion relai et bouton
// si allumé et plus de 5s alors eteindre
// si eteind alors allumer
  stateButton = digitalRead(pinButton);

  if(stateButton == HIGH && previous == LOW && millis() - time > delaiButton ) {
    previous = HIGH;
    time = millis();
    Serial << F("Bouton on ");
    if(stateRelay == HIGH){
    digitalWrite(RelayOFF, HIGH); // Relay OFF //
    delay(7);
    digitalWrite(RelayOFF, LOW);
    stateRelay = LOW;
    Serial << F("Relai off ");
    } else {
    digitalWrite(RelayON, HIGH);  // Relay ON //
    delay(7);
    digitalWrite(RelayON, LOW);
    stateRelay = HIGH;
    Serial << F("Relai on ");
    }
   }
  if(stateButton == LOW){
    previous=LOW;
    time = millis();
  }
// gestion horloge 
    static time_t tLast;
    time_t t;
    tmElements_t tm;

    //check for input to set the RTC, minimum length is 12, i.e. [1|2|3],yy,m,d,h,m,s
    // T pour mise a l heure
    // A ou B pour Alarmes
    if (Serial.available() >= 12) {
        //note that the tmElements_t Year member is an offset from 1970,
        //but the RTC wants the last two digits of the calendar year.
        //use the convenience macros from Time.h to do the conversions.
        int cmd = Serial.parseInt();
        if (cmd< 1 || cmd>9) {
          Serial << F("1 cmd temps ex 1,14,10,22,17,16,30");
          Serial << endl;
          Serial << F("2 cmd Alarm Hour ON  2,17,16,30 on every 17h 16:30");
          Serial << endl;
          Serial << F("3 cmd Alarm Hour OFF 3,17,16,30 on every 17h 16:30");
          Serial << endl;
          Serial << F("4 cmd Alarm Date ON  4,17,16,30,22 on every 17h 16:30 22_of_the_month");
          Serial << endl;
          Serial << F("5 cmd Alarm Date OFF 5,17,18,30,22 off every 17h 18:00 22_of_the_month");
          Serial << endl;
          Serial << F("6 cmd delay ON  in seconde before on ex 6,300,                on in 300 s");
          Serial << endl;
          Serial << F("7 cmd delay OFF in seconde before off ex 7,300,                off in 300 s");
          Serial << endl;
        }
        int y = Serial.parseInt();
        if (y >= 100 && y < 1000 && cmd==1)
            Serial << F("Error: Year must be two digits or four digits!") << endl;
        else {
            if (y >= 1000 && cmd==1)
                tm.Year = CalendarYrToTm(y);
            else    //(y < 100)
               if(cmd==1) {   // set date time //
                  tm.Year = y2kYearToTm(y);
                  tm.Month = Serial.parseInt();
                  tm.Day = Serial.parseInt();
                  tm.Hour = Serial.parseInt();
                  tm.Minute = Serial.parseInt();
                  tm.Second = Serial.parseInt();
                  t = makeTime(tm);
                  RTC.set(t);        //use the time_t value to ensure correct weekday is set
                  setTime(t);        
                  Serial << F("RTC set to: ");
                  printDateTime(t);
               }
               else {
    /*Alarm_Types defined in RTC3232.h.............................................
  
    ALM1_EVERY_SECOND 
    ALM1_MATCH_SECONDS
    ALM1_MATCH_MINUTES     //match minutes *and* seconds
    ALM1_MATCH_HOURS       //match hours *and* minutes, seconds
    ALM1_MATCH_DATE        //match date *and* hours, minutes, seconds
    ALM1_MATCH_DAY         //match day *and* hours, minutes, seconds
    ALM2_EVERY_MINUTE 
    ALM2_MATCH_MINUTES     //match minutes
    ALM2_MATCH_HOURS       //match hours *and* minutes
    ALM2_MATCH_DATE       //match date *and* hours, minutes
    ALM2_MATCH_DAY        //match day *and* hours, minutes
  */
   //setAlarm Format (Alarm_Type, seconds, minutes, hours,  date the month or day of the week))
   //RTC.setAlarm(ALM2_EVERY_MINUTE, 0,0,0,0);  //
    //Set Alarm1 for 12:34:56 on Sunday
    //RTC.setAlarm(ALM1_MATCH_DAY, 56, 27, 16, dowThursday);//
    //RTC.setAlarm(ALM1_MATCH_SECONDS,30,0,0,0);//
 //    RTC.setAlarm(ALM1_MATCH_DATE,30,0,0,0);//
 
 //Set Alarm1 for 12:34:56 on Sunday
 // RTC.setAlarm(ALM1_MATCH_DAY, 56, 34, 12, dowSunday); //
 
 //Set Alarm2 for 12:34 on the 4th day of the month
 // RTC.setAlarm(ALM1_MATCH_DATE, 34, 12, 4); //
                  tm.Hour = y;
                  tm.Minute = Serial.parseInt();
                  tm.Second = Serial.parseInt();
                  tm.Day = Serial.parseInt();

                 Serial << tm.Hour <<  tm.Minute << tm.Second << tm.Day  ;
                 Serial << endl;

                  if(cmd==9) {  // Reset Alarme 1 2 //
                     RTC.alarmInterrupt(ALARM_1, false);      //assert the INT pin when Alarm1 occurs.
                     RTC.alarmInterrupt(ALARM_2, false);     // Alarm2
                     RTC.setAlarm(ALM1_MATCH_DATE, 0,0,0,0); 
                     RTC.setAlarm(ALM2_MATCH_DATE, 0,0,0,0); 
                     Serial << F("reset alarms " );
                  }
                  if(cmd==2) {  // Alarme  1 ON ALM1_MATCH_HOURS //
                     RTC.alarmInterrupt(ALARM_1, true);
                     RTC.setAlarm(ALM1_MATCH_HOURS, tm.Second, tm.Minute, tm.Hour, 0 );
                     Serial << F("ALM1 HOURS on ")  ;
                  }
                  if(cmd==3) {  // Alarme 2 OFF ON ALM2_MATCH_HOURS //
                     RTC.alarmInterrupt(ALARM_2, true);
                     RTC.setAlarm(ALM2_MATCH_HOURS, tm.Minute, tm.Hour,0); 
                     Serial << F("ALM2 HOURS for off ")  ; 
                  }
                  if(cmd==4) {  // Alarme  1 ON ALM1_MATCH_DATE //
                     RTC.alarmInterrupt(ALARM_1, true);
                     RTC.setAlarm(ALM1_MATCH_DATE, tm.Second, tm.Minute, tm.Hour, tm.Day); 
                     Serial << F("ALM1 DATE on ");  
                 }
                 if(cmd==5) {  // Alarme 2 OFF ON ALM2_MATCH_DATE //
                     RTC.alarmInterrupt(ALARM_2, true);
                     RTC.setAlarm(ALM2_MATCH_DATE, tm.Second, tm.Minute, tm.Hour, tm.Day); 
                     Serial << F("ALM2 DATE for off "); 
                 }
                 if(cmd==7) {  // delay secondes before stop //
                    if(y<=120){
                    delay(y*1000);
                    Serial << F("delay, OFF");
                    digitalWrite(RelayOFF, HIGH); // Relay OFF //
                    delay(7);
                    digitalWrite(RelayOFF, LOW);
                    stateRelay = LOW;
                    }
                 }
                 if(cmd==6) {  // delay before on //
                    if(y<=120){
                    delay(y*1000);
                    Serial << F("delay ON");
                    digitalWrite(RelayON, HIGH); // Relay ON //
                    delay(7);
                    digitalWrite(RelayON, LOW);
                    stateRelay = HIGH;
                    }
                 }
                 Serial << endl;
            }
            //dump any extraneous input
            while (Serial.available() > 0) Serial.read();
        }
    }
    
    // capteur humidité //
    int chk = DHT.read22(DHT22_PIN);
    //
    
    t = now();
    if (t != tLast) {
        tLast = t;
        printDateTime(t);
        // if (second(t) == 0 || second(t) == 30) { //
            float c = RTC.temperature() / 4.;
            //float f = c * 9. / 5. + 32.;//
            //Serial << F("  ") << c << F(" C  ") << f << F(" F");//
            Serial << F("  ") << c << F(" C  ");
        //}//
        //if (chk == DHTLIB_OK) {
            Serial << DHT.humidity <<  F(" ") << DHT.temperature << F(" C  ");
        //}
        Serial << endl;
        //
        // programmation en dur en attendant mieux ;) //
        //
        //if ( hour(t) == 11 && minute(t) == 0 && ( second(t) == 0 || second(t)==30) ) {
        //  digitalWrite(RelayON, HIGH);  // Relay ON //
        //  delay(7);
        //  digitalWrite(RelayON, LOW);
        //  stateRelay = HIGH;
        //}
        if ( hour(t) == 8 && minute(t) == 0 && ( second(t) == 0 || second(t)==30) ) {
          digitalWrite(RelayON, HIGH);  // Relay ON //
          delay(7);
          digitalWrite(RelayON, LOW);
          stateRelay = HIGH;
        }
        if ( hour(t) == 16 && minute(t) == 0 && ( second(t) == 0 || second(t)==30)  ) {
          digitalWrite(RelayON, HIGH);  // Relay ON //
          delay(7);
          digitalWrite(RelayON, LOW);
          stateRelay = HIGH;
        }
        if ( hour(t) == 18 && minute(t) == 0 && ( second(t) == 0 || second(t)==30) ) {
          digitalWrite(RelayON, HIGH);  // Relay ON //
          delay(7);
          digitalWrite(RelayON, LOW);
          stateRelay = HIGH;
        }
        if ( hour(t) == 10 && minute(t) == 0 && ( second(t) == 0 || second(t)==30) ) {
          digitalWrite(RelayOFF, HIGH);  // Relay ON //
          delay(7);
          digitalWrite(RelayOFF, LOW);
          stateRelay = LOW;
        }
        if ( hour(t) == 22 && minute(t) == 0 && ( second(t) == 0 || second(t)==30) ) {
          digitalWrite(RelayOFF, HIGH);  // Relay ON //
          delay(7);
          digitalWrite(RelayOFF, LOW);
          stateRelay = LOW;
        }
        // FIN
    }
    if ( rtcok==1 && RTC.alarm(ALARM_1) ) {
      Serial << F("Alarm 1, ON");
      Serial << endl;
      digitalWrite(RelayON, HIGH);  // Relay ON //
      delay(7);
      digitalWrite(RelayON, LOW);
      stateRelay = HIGH;
    }
    if ( rtcok==1 && RTC.alarm(ALARM_1) ) {
      Serial << F("Alarm 1, ON");
      Serial << endl;
      digitalWrite(RelayON, HIGH);  // Relay ON //
      delay(7);
      digitalWrite(RelayON, LOW);
      stateRelay = HIGH;
    }
   if ( rtcok==1 && RTC.alarm(ALARM_2) ) {
      Serial << F("Alarm 2, OFF");
      Serial << endl;
      digitalWrite(RelayOFF, HIGH); // Relay OFF //
      delay(7);
      digitalWrite(RelayOFF, LOW);
      stateRelay = LOW;
    }
    // delais pour la lecture du DHT //
    delay(1000);
}
